{+------------------------------------------------------------
 | Unit <!UNITNAME>
 |
 | Version: 1.0  Created: <!DATE>
 |               Last Modified: <!DATE>
 | Author : GenTypedLists Version <!VERSION>
 | Project: General utilities
 | Description:
 |   
 +------------------------------------------------------------}
Unit <!UNITNAME>;
Interface

Uses Classes, AbstractTypedList <!USESCLAUSE>;

<!DEFINES>
Type
  <!DATATYPE>SortCompare = Function( Const item1, item2: <!DATATYPE> ): Integer;

  <!LISTCLASSNAME> = Class( TAbstractTypedList )
  Private
    Procedure Put( index: Integer; Const Item: <!DATATYPE> );
    Function  Get( index: Integer ): <!DATATYPE>;
    Procedure FreeItem( index: Integer );
    Procedure QuickSort(L, R: Integer; Compare: <!DATATYPE>SortCompare );

  Public
    Function Clone: <!LISTCLASSNAME>;

    Procedure Assign( source: TPersistent ); override;
    Function  Add(Const Item: <!DATATYPE>): Integer;
    Procedure Insert(Index: Integer; Const Item: <!DATATYPE>);

    Procedure Clear; override;
    Procedure Delete(Index: Integer); override;
    Function  Detach(Index: Integer): <!DATATYPE>; 

    Function LinearSearch( Const Item: <!DATATYPE>; 
                           Compare: <!DATATYPE>SortCompare): Integer;
    Function BinarySearch( Const Item: <!DATATYPE>; 
                           Compare: <!DATATYPE>SortCompare): Integer;
    Procedure Sort(Compare: <!DATATYPE>SortCompare);

    Property Items[Index: Integer]: <!DATATYPE> read Get write Put; default;
  End;

{$IFDEF GTL_USECOMPAREPROC} 
Function <!DATATYPE>Compare( Const item1, item2: <!DATATYPE> ): Integer;
{$ENDIF} 

Implementation

{$IFDEF GTL_USEPOINTERS} 
Type
    <!DATATYPE>_Ptr = ^<!DATATYPE>;
{$ENDIF} 

{$IFDEF GTL_USECOMPAREPROC} 
{+------------------------------------------------------------
 | Function <!DATATYPE>Compare
 |
 | Parameters :
 |   item1, item2: the two <!DATATYPE> items to compare
 | Returns    : 
 |   0 if both items are equal, < 0 if item1 is smaller than item2,
 |   > 0 if item1 is larger than item2
 | Error Conditions: none
 | Created: <!DATE> by GenTypedLists Version <!VERSION>
 +------------------------------------------------------------}
Function <!DATATYPE>Compare( Const item1, item2: <!DATATYPE> ): Integer;
  Begin 
    {  MODIFY THIS IMPLEMENTATION AS APPROPRIATE!  }
    If item1 < item2 Then 
      Result := -1
    Else If item1 > item2 Then 
      Result := 1
    Else
      Result := 0;
  End; { <!DATATYPE>Compare }
{$ENDIF} 

{+-----------------------------
 | Methods of <!LISTCLASSNAME> 
 +----------------------------}

{+------------------------------------------------------------
 | Function <!LISTCLASSNAME>.Clone: <!LISTCLASSNAME>;
 |
 | Returns : a deep copy of this list instance.
 | Call method: static
 | Visibility : public
 | Description:
 |   Returns a new instance of the list class and copies the 
 |   data from this list into it. 
 | Error Conditions: none
 | Created: <!DATE> by GenTypedLists Version <!VERSION>
 +------------------------------------------------------------}
Function <!LISTCLASSNAME>.Clone: <!LISTCLASSNAME>;
  Begin
    Result := <!LISTCLASSNAME>.Create;
    Result.Assign( self );
  End; { <!LISTCLASSNAME>.Clone }

{+------------------------------------------------------------
 | Procedure <!LISTCLASSNAME>.Assign
 |
 | Parameters :
 |   source: list instance to copy
 | Call method: virtual, overriden
 | Visibility : public
 | Description:
 |   Copies the data from the source into this instance, which
 |   is cleared first. This is a deep copy, unless the
 |   list contains a pointer type other than an object. If the
 |   list contains objects and owns
 |   the objects (GTL_OWNSOBJECTS defined) then the object 
 |   class MUST have a public Clone method, or a compiler error
 |   will result! If the list does not own the objects the copy
 |   is shallow, the new list stores references to the same objects
 |   as aList.
 | Error Conditions: 
 |   If source is not of this list class the inherited Assign is
 |   called and that will raise an exception!
 | Created: <!DATE> by GenTypedLists Version <!VERSION>
 +------------------------------------------------------------}
Procedure <!LISTCLASSNAME>.Assign( source: TPersistent ); 
  Var
    i: Integer;
    aList : <!LISTCLASSNAME>;
  Begin
    If Assigned( source ) Then Begin 
      If source Is <!LISTCLASSNAME> Then Begin 
        aList := <!LISTCLASSNAME>( source );
        Clear;
        Capacity := aList.Count;
        For i := 0 To aList.LastIndex Do Begin 
          {$IFDEF GTL_OWNSOBJECTS}
          Insert( count, aList.Items[ i ].Clone );
          {$ELSE}
          Insert( count, aList.Items[ i ] );
          {$ENDIF}
        End; { For }
      End { If }
      Else
        inherited Assign( source );
    End; { If }
  End; { <!LISTCLASSNAME>.Assign }

{+------------------------------------------------------------
 | Function <!LISTCLASSNAME>.Add
 |
 | Parameters :
 |   Item to add. If this is a pointer type the list will store
 |   the pointer as is!
 | Returns    : the index of the item
 | Call method: static
 | Visibility : public
 | Description:
 |   Adds the passed item to the end of the list and returns the
 |   index of the item.
 | Error Conditions: 
 |   We may run out of memory here, which will cause an exception.
 | Created: <!DATE> by GenTypedLists Version <!VERSION>
 +------------------------------------------------------------}
Function <!LISTCLASSNAME>.Add(Const Item: <!DATATYPE>): Integer;
  Begin
    Result := Count;
    Insert( Result, Item );
  End; { <!LISTCLASSNAME>.Add }

{+------------------------------------------------------------
 | Procedure <!LISTCLASSNAME>.Insert
 |
 | Parameters :
 |   Index: index of the item before which to insert the new item. 
 |          If Index >= Count the item will be appended to the list.
 |   Item : item to insert
 | Call method: static
 | Visibility : public
 | Description:
 |   Inserts the passed item into the list. If the data type is 
 |   larger than 4 bytes memory for the Item is allocated on the 
 |   heap, the item is copied into it and the pointer is stored
 |   in the list, otherwise the data is stored directly, typecast
 |   to a pointer.
 | Error Conditions: 
 |   We may run out of memory here, which will cause an exception.
 | Created: <!DATE> by GenTypedLists Version <!VERSION>
 +------------------------------------------------------------}
Procedure <!LISTCLASSNAME>.Insert(Index: Integer; Const Item: <!DATATYPE>);
  Begin
    If Index > Count Then 
      Index := Count;
    Storage.Insert( index, Nil );
    Put( index, Item );
  End; { <!LISTCLASSNAME>.Insert }

{+------------------------------------------------------------
 | Procedure <!LISTCLASSNAME>.Clear
 |
 | Parameters : none
 | Call method: virtual, overriden
 | Visibility : public
 | Description:
 |   Clears the list, freeing memory for the items if necessary.
 | Error Conditions: none
 | Created: <!DATE> by GenTypedLists Version <!VERSION>
 +------------------------------------------------------------}
Procedure <!LISTCLASSNAME>.Clear; 
  {$IFDEF GTL_OWNSOBJECTS} {$DEFINE GTL_MUSTFREE} {$ENDIF}
  {$IFDEF GTL_USEPOINTERS} {$DEFINE GTL_MUSTFREE} {$ENDIF}
  {$IFDEF GTL_MUSTFREE} 
  Var
    i: Integer;
 {$ENDIF} 
  Begin
    {$IFDEF GTL_MUSTFREE} 
      For i := LastIndex Downto 0 Do  
        FreeItem( i );
      {$UNDEF GTL_MUSTFREE }
    {$ENDIF} 
    inherited Clear;
  End; { <!LISTCLASSNAME>.Clear }

{+------------------------------------------------------------
 | Procedure <!LISTCLASSNAME>.FreeItem
 |
 | Parameters : index of item to free
 | Call method: static
 | Visibility : private
 | Description:
 |   Frees the memory for the item at index, if required.
 | Error Conditions: 
 |   A invalid index will raise a EListError exception!
 | Created: <!DATE> by GenTypedLists Version <!VERSION>
 +------------------------------------------------------------}
Procedure <!LISTCLASSNAME>.FreeItem( index: Integer );
  {$IFDEF GTL_USEPOINTERS} 
  Var
    pitem: <!DATATYPE>_Ptr;
  {$ENDIF}
  Begin
    {$IFDEF GTL_OWNSOBJECTS}
      Items[ Index ].Free;
    {$ELSE}
      {$IFDEF GTL_USEPOINTERS} 
        pItem := <!DATATYPE>_Ptr( Storage[ Index ] );
        If Assigned( pItem ) Then 
          Dispose( pItem );
      {$ENDIF} 
    {$ENDIF}
  End; { <!LISTCLASSNAME>.FreeItem }

{+------------------------------------------------------------
 | Procedure <!LISTCLASSNAME>.Delete
 |
 | Parameters :
 |   Index: index of item to delete.
 | Call method: virtual, overridden
 | Visibility : public
 | Description:
 |   Deletes the item at index from the list, freeing memory for
 |   it if necessary.
 | Error Conditions: 
 |   A invalid index will raise a EListError exception!
 | Created: <!DATE> by GenTypedLists Version <!VERSION>
 +------------------------------------------------------------}
Procedure <!LISTCLASSNAME>.Delete(Index: Integer); 
  Begin
    FreeItem( Index );
    inherited Delete( Index );
  End; { <!LISTCLASSNAME>.Delete }

{+------------------------------------------------------------
 | Function  <!LISTCLASSNAME>.Detach
 |
 | Parameters : index of item to remove from the list
 | Returns    : the removed item
 | Call method: static
 | Visibility : public
 | Description:
 |   Returns the item at index and then deletes the entry from
 |   the list. If the list stores objects the object will of 
 |   course *not* be freed, the list relegates ownership of the
 |   data to the caller.
 | Error Conditions: 
 |   A invalid index will raise a EListError exception!
 | Created: <!DATE> by GenTypedLists Version <!VERSION>
 +------------------------------------------------------------}
Function  <!LISTCLASSNAME>.Detach(Index: Integer): <!DATATYPE>; 
  Begin
    Result := Items[ Index ];
    {$IFDEF GTL_USEPOINTERS} 
    Delete( index );
    {$ELSE}
    inherited Delete( index );
    {$ENDIF} 
  End; { <!LISTCLASSNAME>.Detach }

{+------------------------------------------------------------
 | Procedure <!LISTCLASSNAME>.Put
 |
 | Property   : Items ( write )
 | Call method: static
 | Visibility : private
 | Description:
 |   Frees memory for the item at index, if required, and stores
 |   the passed item in that slot of the list. If the data type is 
 |   larger than 4 bytes memory for the Item is allocated on the 
 |   heap, the item is copied into it and the pointer is stored
 |   in the list, otherwise the data is stored directly, typecast
 |   to a pointer.
 | Error Conditions: 
 |   A invalid index will raise a EListError exception!
 | Created: <!DATE> by GenTypedLists Version <!VERSION>
 +------------------------------------------------------------}
Procedure <!LISTCLASSNAME>.Put( index: Integer; Const Item: <!DATATYPE> );
  {$IFDEF GTL_USEPOINTERS} 
  Var
    pTemp: <!DATATYPE>_Ptr;
  {$ENDIF} 
  Begin
    FreeItem( index );
    {$IFDEF GTL_USEPOINTERS} 
    New( pTemp );
    Try
      pTemp^ := Item;
      Storage.Items[ index ]:= pTemp;
    Except
      Dispose( pTemp );
      raise
    End;
    {$ELSE} 
    Storage.Items[ Index ]:= Pointer( Item );
    {$ENDIF} 
  End; { <!LISTCLASSNAME>.Put }

{+------------------------------------------------------------
 | Function  <!LISTCLASSNAME>.Get
 |
 | Property   : Items ( read )
 | Call method: static
 | Visibility : private
 | Description:
 |   Returns the item at Index in the list.
 | Error Conditions: 
 |   A invalid index will raise a EListError exception!
 | Created: <!DATE> by GenTypedLists Version <!VERSION>
 +------------------------------------------------------------}
Function  <!LISTCLASSNAME>.Get( index: Integer ): <!DATATYPE>;
  Begin
    {$IFDEF GTL_USEPOINTERS} 
    Result := <!DATATYPE>_Ptr( Storage.Items[ index ] )^
    {$ELSE} 
    Result := <!DATATYPE>( Storage.Items[ index ] );
    {$ENDIF} 
  End; { <!LISTCLASSNAME>.Get }

{+------------------------------------------------------------
 | Function <!LISTCLASSNAME>.LinearSearch
 |
 | Parameters :
 |   Item: item to search for
 |   Compare: compare method to use
 | Returns    : the index of the item, or -1, if the item is not
 |              found.
 | Call method: static
 | Visibility : public
 | Description:
 |   Performs a linear search over the list and stops at the first
 |   item that matches the passed one.
 | Error Conditions: none
 | Created: <!DATE> by GenTypedLists Version <!VERSION>
 +------------------------------------------------------------}
Function <!LISTCLASSNAME>.LinearSearch( Const Item: <!DATATYPE>; 
                       Compare: <!DATATYPE>SortCompare): Integer;
  Var
    i: Integer;
  Begin
    Assert( Assigned( Compare ));
    Result := -1;
    For i := 0 To LastIndex Do Begin 
      If Compare( Item, Items[ i ] ) = 0 Then Begin
        Result := i;
        Break;
      End; { If }
    End; { For }
  End; { <!LISTCLASSNAME>.LinearSearch }

{+------------------------------------------------------------
 | Function <!LISTCLASSNAME>.BinarySearch
 |
 | Parameters :
 |   Item: item to search for
 |   Compare: compare method to use
 | Returns    : the index of the item, or -1, if the item is not
 |              found.
 | Call method: static
 | Visibility : public
 | Description:
 |   Performs a binary search over the list and stops at the first
 |   item that matches the passed one. The list needs to be sorted
 |   for this kind of find to work and the same Compare function 
 |   needs to be used for sort and search! If the list contains duplicate 
 |   items binary search will find one of them but not necessarily 
 |   the first!
 |   The implementation has not been optimized for speed in any way!
 | Error Conditions: none
 | Created: <!DATE> by GenTypedLists Version <!VERSION>
 +------------------------------------------------------------}
Function <!LISTCLASSNAME>.BinarySearch( Const Item: <!DATATYPE>; 
                       Compare: <!DATATYPE>SortCompare): Integer;
  Var
    first, last, pivot, res: Integer;
  Begin
    Assert( Assigned( Compare ));
    Result := -1;
    If count = 0 Then Exit;

    first := 0;
    last  := LastIndex;
    Repeat 
      pivot := ( first + last ) div 2;
      res := Compare( Item, Items[ pivot ] );
      If res = 0 Then Begin 
        { Found the item, return its index and exit. }
        Result := pivot;
        Break;
      End { If } 
      Else If res > 0 Then Begin 
        { Item is larger than item at pivot }
        first := pivot + 1;
      End { If }
      Else Begin 
        { Item is smaller than item at pivot } 
        last := pivot - 1;
      End;
    Until last < first;
  End; { <!LISTCLASSNAME>.BinarySearch }

Procedure <!LISTCLASSNAME>.QuickSort(L, R: Integer; Compare: <!DATATYPE>SortCompare );
  Var
    I, J: Integer;
    P: <!DATATYPE>;
  Begin
    Repeat
      I := L;
      J := R;
      P := Items[(L + R) shr 1];
      Repeat
        While Compare(Items[I], P) < 0 Do Inc(I);
        While Compare(Items[J], P) > 0 Do Dec(J);
        If I <= J Then
        Begin
          Exchange( I, J );
          Inc(I);
          Dec(J);
        End;
      Until I > J;
      If L < J Then QuickSort(L, J, Compare);
      L := I;
    Until I >= R;
  End; { <!LISTCLASSNAME>.QuickSort }

{+------------------------------------------------------------
 | Procedure <!LISTCLASSNAME>.Sort
 |
 | Parameters : function to use to compare items
 | Call method: static
 | Visibility : public
 | Description:
 |   Performs a quicksort on the list. The sort code is modified
 |   from TList.Sort.
 | Error Conditions: none
 | Created: <!DATE> by GenTypedLists Version <!VERSION>
 +------------------------------------------------------------}
Procedure <!LISTCLASSNAME>.Sort(Compare: <!DATATYPE>SortCompare);
  Begin
    Assert( Assigned( Compare ));
    If Count > 1 Then 
      QuickSort(0, LastIndex, Compare);
  End; { <!LISTCLASSNAME>.Sort }

Initialization
End. { <!UNITNAME> }

